<?php

namespace Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Inflightor\Util\MessagesProcedureInflightor;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\DBAL\Connection;

class PlanejamentoProducaoController extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = "";
    protected $dataSourceNameChecklist = "cklistprodinf";
    protected $dataSourceNameReceitas = "reccklistprodinf";
    protected $dataSourceNameDestinos = "destcklistprodinf";
    
    public function __construct($crudService, $parameterBag,  $environment, $dataSourceManager, $MessagesProcedureInflightor, $GerarChecklistProcedure, $PlanejamentoProducaoService, $entityManager, $RateioService) {
        $this->crudService = $crudService;
        $this->parameterBag=$parameterBag;
        $this->environment=$environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->MessagesProcedureInflightor = $MessagesProcedureInflightor;
        $this->GerarChecklistProcedure = $GerarChecklistProcedure;
        $this->PlanejamentoProducaoService = $PlanejamentoProducaoService;
        $this->entityManager = $entityManager;
        $this->RateioService = $RateioService;
    }

    public function geraChecklistProd(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $row['P_CDOPERADOR'] = $this->environment->getCdOperador();
            $row['P_NRORG'] = $this->environment->getNrOrgTrab();
            $resultado = $this->GerarChecklistProcedure->executeProcedure($row['P_CDFILIAL'], $row['P_DTPRODUCAO'], $row['P_CDSETOR'], $row['P_CDOPERADOR'], $row['P_NRORG']);  
            if($resultado['P_NRLISTAERROS'] != null) {
                $this->MessagesProcedureInflightor->getErrorMessages($row['P_CDFILIAL'], $resultado['P_NRLISTAERROS'], $response);
                $connection->rollback();
                return;
            }
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollback();
            $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
        }
    }

    public function deleteChecklist(DTO\Request\DataSet $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
                foreach ($rows as $row) {
                    //Exclusão completa dos Checklists selecionados
                    $params = array(
                        'CDFILIAL'        => $row['CDFILIAL'],
                        'NRCKLISTPRODINF' => $row['NRCKLISTPRODINF']
                    );
                    $receitas = $this->PlanejamentoProducaoService->find($params, 'SELECT_RECEITAS');
                    if(!empty($receitas)){
                        //Deleta todos os Destinos e Justificativas da Receita
                        foreach ($receitas as $receita) {
                            $paramsDestino = array(
                                'CDFILIAL' => $row['CDFILIAL'],
                                'NRRECCKLISTPRODINF' => $receita['NRRECCKLISTPRODINF']
                            );
                            $this->PlanejamentoProducaoService->delete($paramsDestino, 'DELETE_DESTINOS');
                            $this->PlanejamentoProducaoService->delete($paramsDestino, 'DELETE_JUSREJDESP');
                        }
                        $this->PlanejamentoProducaoService->delete($params, 'DELETE_RECEITAS');
                    }
                    $this->PlanejamentoProducaoService->delete($params, 'DELETE_CHECKLIST');
                }
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollback();
            $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
        }
    }

    public function saveEditDestinos(DTO\Request\DataSet $request, DTO\Response $response){
        $data = $request->getDataSet();
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
                //Salva as edições feitas no destino
                $this->dataSourceName = $this->dataSourceNameDestinos; 
                parent::save($request, $response);
                //Find em tdoos os destinos da Receita para salvar a soma total de sua quantidade
                $paramsDestino = array(
                    'CDFILIAL' => $data->getRows()[0]['CDFILIAL'],
                    'NRRECCKLISTPRODINF' => $data->getRows()[0]['NRRECCKLISTPRODINF']
                );
                $destinos = $this->PlanejamentoProducaoService->find($paramsDestino, 'SELECT_SUM_QTD_DESTINOS');
                $novaQuantidade = $destinos[0]['SUM_QTD'];
                $paramsReceita = array(
                    'CDFILIAL' => $data->getRows()[0]['CDFILIAL'],
                    'NRRECCKLISTPRODINF' => $data->getRows()[0]['NRRECCKLISTPRODINF'],
                    'QTRECEITAINF' => $novaQuantidade
                );
                $this->PlanejamentoProducaoService->update($paramsReceita, 'UPDATE_RECEITA');
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollback();
            $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
        }
    }

    public function saveEditReceitas(DTO\Request\DataSet $request, DTO\Response $response){
        $data = $request->getDataSet();
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
                //Salva todas as edições feitas nas Receitas
                $this->dataSourceName = $this->dataSourceNameReceitas; 
                parent::save($request, $response);
                //Apos salvar, aplica rotina de rateio caso encontre algum destino filho da Receita
                foreach ($request->getDataSet()->getRows() as $row) {
                    //Rotina de Rateio para cada Receita editada
                    $paramsDestino = array(
                        'CDFILIAL' => $row['CDFILIAL'],
                        'NRRECCKLISTPRODINF' => $row['NRRECCKLISTPRODINF']
                    );
                    $destinos = $this->PlanejamentoProducaoService->find($paramsDestino, 'SELECT_DESTINOS');
                    if(count($destinos)){
                        //Caso encontre destinos aplica RATEIO nas quantidades
                        //Informa quais as colunas origem e quais colunas sofreram RATEIO
                        $this->RateioService->setOriginColumns("QTRECEITAINF");
                        $this->RateioService->setAffectedColumns("QTRECEITAINF");
                        //Calcula o rateio baseado na Row origem e os destinos influenciados
                        //retorna os destinos com as colunas já modificadas e prontas para salvar
                        $novosDestinos = $this->RateioService->calcRateio($row, $destinos);
                        //Salva as alterações feitas pelo RATEIO
                        $this->effectSave($novosDestinos, $response);
                    }
                }
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollback();
            $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
        }
    }
    private function effectSave($destinos, $response){
        $destinosSave = $this->fixesColumnsToSave($destinos);
        $dataSet = new DataSet($this->dataSourceNameDestinos, $destinosSave);
        $request = new DTO\Request\DataSet($dataSet, '','','');
        $rowToSave = $this->crudService->getRowsToSave($request, $this->dataSourceNameDestinos);
        $this->crudService->save($this->dataSourceNameDestinos, $rowToSave);
    }
    private function fixesColumnsToSave($destinos){
        foreach ($destinos as &$row) {
            $row['__is_new'] = false;
            $row['changed'] = true;
        }
        return $destinos;
    }
}